from datetime import datetime
from re import search, sub

import requests
from bs4 import BeautifulSoup as bs
from requests.exceptions import ConnectTimeout


class X1337:
    def __init__(self, user_agent):
        self.url = "https://1377x.to"
        self.search = "/search/##"
        self.delimiter = "+"
        self.sort = "/sort-search/##/@@/desc"
        self.sort_type = {
            "age": "time",
            "size": "size",
            "seed": "seeders",
            "leech": "leechers",
        }
        self.page = "/@@/"
        self.user_agent = {"User-Agent": user_agent}

    def _search_torrents(self, query, pages, sort=None):
        query_delim = sub(r"\s", self.delimiter, query)
        url_attach = (
            sub("@@", self.sort_type[sort], sub("##", query_delim, self.sort))
            if sort in self.sort_type
            else sub("##", query_delim, self.search)
        )
        torrents_per_page = [
            bs(
                requests.get(
                    f"{self.url}{url_attach}{sub('@@', str(i + 1), self.page)}",
                    headers=self.user_agent,
                    timeout=3,
                    verify=False,
                ).text,
                "html.parser",
            ).findAll("td")
            for i in range(pages)
        ]
        return sum(torrents_per_page, start=[])

    def build_list(self, query, pages, sort=None):
        try:
            search_list = self._search_torrents(query, pages, sort)
        except ConnectTimeout:
            return []
        singles = [search_list[i * 6 : i * 6 + 6] for i in range(len(search_list) // 6)]
        torrents = list()
        for torrent in singles:
            name = torrent[0].text
            category, link_page = [a.get("href") for a in torrent[0].findAll("a")]
            category = search(r"sub\/(\w+)\/", category).group(1)
            seed, leech, time, size = [t.text for t in torrent[1:5]]
            torrents.append(
                [
                    "1337x",
                    name,
                    f"{self.url}{link_page}",
                    self._date_converter(time),
                    category.title(),
                    sub(r"\d+$", "", size),
                    int(sub(",", "", seed)),
                    int(sub(",", "", leech)),
                ]
            )
        return torrents

    def _date_converter(self, site_date):
        if search(r"(am|pm)", site_date):
            return datetime.now().strftime("%d.%m.%Y")
        month, day, year = site_date.split()
        months = {
            "Jan.": 1,
            "Feb.": 2,
            "Mar.": 3,
            "Apr.": 4,
            "May.": 5,
            "Jun.": 6,
            "Jul.": 7,
            "Aug.": 8,
            "Sep.": 9,
            "Oct.": 10,
            "Nov.": 11,
            "Dec.": 12,
        }
        day = int(sub(r"\D+", "", day))
        month = months.get(month, 1)
        return datetime(2000 + int(year[1:]), month, day).strftime("%d.%m.%Y")

    def get_magnet_link(self, torrent_page):
        soup = bs(
            requests.get(torrent_page, allow_redirects=True, verify=False).text,
            "html.parser",
        )
        try:
            return [
                a.get("href")
                for a in soup.findAll("a")
                if search("magnet", a.get("href"))
            ][0]
        except IndexError:
            return None



from functools import reduce
from re import search, sub

import requests
from bs4 import BeautifulSoup as bs
from requests.exceptions import ConnectTimeout, ReadTimeout


class CorsaroNero:
    def __init__(self, user_agent):
        self.url = "https://ilcorsaronero.link"
        self.search = "/advsearch.php?search=##"
        self.delimiter = "+"
        self.sort = "/advsearch.php?search=##&&order=@@&by=DESC"
        self.sort_type = {
            "age": "data",
            "size": "size",
            "seed": "seeds",
            "leech": "peers",
        }
        self.page = "&page=@@"
        self.user_agent = {"User-Agent": user_agent}

    def _search_torrents(self, query, pages, sort=None):
        url_attach = (
            sub(
                "@@",
                self.sort_type[sort],
                sub("##", sub(r"\s", self.delimiter, query), self.sort),
            )
            if sort and sort in self.sort_type
            else sub("##", sub(r"\s", self.delimiter, query), self.search)
        )
        return reduce(
            lambda x, y: x + y,
            [
                bs(
                    requests.get(
                        "{}{}{}".format(
                            self.url, url_attach, sub("@@", str(i), self.page)
                        ),
                        headers=self.user_agent,
                        timeout=3,
                        verify=False,
                    ).text,
                    "html.parser",
                ).findAll("td")[18:-3]
                for i in range(pages)
            ],
            [],
        )

    def build_list(self, query, pages, sort=None):
        try:
            search_list = self._search_torrents(query, pages, sort)
        except (ConnectTimeout, ReadTimeout):
            return []
        search_list = [x for x in search_list if x.text and not x.get("colspan")]
        singles = [search_list[i * 6 : i * 6 + 6] for i in range(len(search_list) // 6)]
        torrents = list()
        for torrent in singles:
            category = torrent[0].text
            name = torrent[1].find("a").text
            link_page = sub(
                self.url, "", [a.get("href") for a in torrent[1].findAll("a")][0]
            )
            size = torrent[2].text
            time, seed, leech = [
                "0" if x == "n/a" else x for x in [t.text for t in torrent[3:]]
            ]
            torrents.append(
                [
                    "CorsaroNero",
                    name,
                    f"{self.url}{link_page}",
                    sub(r"(\d\d)$", r"20\1", time),
                    category,
                    sub(r"\d+$", "", size),
                    int(sub(",", "", seed)),
                    int(sub(",", "", leech)),
                ]
            )
        return torrents

    def get_magnet_link(self, torrent_page):
        soup = bs(
            requests.get(torrent_page, allow_redirects=True, verify=False).text,
            "html.parser",
        )
        try:
            return [
                a.get("href")
                for a in soup.findAll("a")
                if search("magnet", str(a.get("href")))
            ][0]
        except IndexError:
            return None



from functools import reduce
from re import search, sub

import requests
from bs4 import BeautifulSoup as bs
from requests.exceptions import ConnectTimeout, ReadTimeout


class TorrentDownloads:
    def __init__(self, user_agent):
        self.url = "https://www.torrentdownloads.me"
        self.search = "/search/?search=##"
        self.delimiter = "+"
        self.sort = None
        self.sort_type = None
        self.page = "&page=@@"
        self.user_agent = {"User-Agent": user_agent}

    def _search_torrents(self, query, pages, sort=None):
        url_attach = sub("##", sub(r"\s", self.delimiter, query), self.search)
        return reduce(
            lambda x, y: x + y,
            [
                bs(
                    requests.get(
                        "{}{}{}".format(
                            self.url, url_attach, sub("@@", str(i + 1), self.page)
                        ),
                        headers=self.user_agent,
                        timeout=3,
                        verify=False,
                    ).text,
                    "html.parser",
                ).findAll("div", {"class": "grey_bar3"})[6:-1]
                for i in range(pages)
            ],
            [],
        )

    def build_list(self, query, pages, sort=None):
        try:
            search_list = self._search_torrents(query, pages, sort)
        except (ConnectTimeout, ReadTimeout):
            return []
        torrents = list()
        for torrent in search_list:
            if not search("No torrents", torrent.text):
                name = torrent.find("p").text
                link_page = [a.get("href") for a in torrent.find("p").findAll("a")][0]
                category = int(
                    search(
                        r"menu_icon(\d+)", torrent.find("p").find("img").get("src")
                    ).group(1)
                )
                leech, seed, size = [
                    s.text
                    for s in torrent.findAll("span")
                    if s.text and not search("mail", s.text)
                ]
                torrents.append(
                    [
                        "TorrentDownloads",
                        name,
                        f"{self.url}{link_page}",
                        "",
                        self._get_category(category),
                        sub(r"\xa0", " ", size),
                        int(sub(",", "", seed)),
                        int(sub(",", "", leech)),
                    ]
                )
        return torrents

    def _get_category(self, category_id):
        categories = {
            4: "Movies",
            5: "Music",
            8: "Television",
            3: "Games",
            7: "Software",
            1: "Anime",
            2: "E-Books",
            9: "Other",
        }
        return categories[category_id] if category_id in categories else "Other"

    def get_magnet_link(self, torrent_page):
        soup = bs(
            requests.get(torrent_page, allow_redirects=True, verify=False).text,
            "html.parser",
        )
        try:
            return [
                a.get("href")
                for a in soup.findAll("a")
                if search("magnet", str(a.get("href")))
            ][0]
        except IndexError:
            return None
